@@ -4,42 +4,16 @@ use warnings;
 use strict;
 use Carp;
 use LWP::UserAgent;
-use Crypt::Rijndael;
-use MIME::Base64;
+# use Crypt::Rijndael;
+# use MIME::Base64;
+use HTML::Tiny;
 
-use version; our $VERSION = qv( '0.6' );
+use version; our $VERSION = qv( '0.7' );
 
 use constant API_SERVER          => 'http://api.recaptcha.net';
 use constant API_SECURE_SERVER   => 'https://api-secure.recaptcha.net';
 use constant API_VERIFY_SERVER   => 'http://api-verify.recaptcha.net';
-use constant API_MAILHIDE_SERVER => 'http://mailhide.recaptcha.net';
-
-# TODO: Find out whether there's a proper code for a server error.
-use constant SERVER_ERROR => 'recaptcha-not-reachable';
-
-my %ENT_MAP = (
-    '&' => '&amp;',
-    '<' => '&lt;',
-    '>' => '&gt;',
-    '"' => '&quot;',
-    "'" => '&apos;',
-);
-
-my %JSON_ESCAPE = (
-    '"'  => '\"',
-    "\n" => '\n',
-    "\t" => '\t',
-    "\r" => '\r',
-);
-
-sub _hash_re {
-    my $hash = shift;
-    my $match = join( '|', map quotemeta, sort keys %$hash );
-    return qr/($match)/;
-}
-
-my $ENT_RE  = _hash_re( \%ENT_MAP );
-my $JSON_RE = _hash_re( \%JSON_ESCAPE );
+use constant SERVER_ERROR        => 'recaptcha-not-reachable';
 
 sub new {
     my $class = shift;
@@ -56,101 +30,18 @@ sub _initialize {
       unless 'HASH' eq ref $args;
 }
 
-# Key validation hook. Currently unused
-sub _check_key {
-    my ( $type, $key ) = @_;
-    return $key;
-}
-
-# Get a UA singleton
-sub _get_ua {
-    my $self = shift;
-    $self->{ua} ||= LWP::UserAgent->new();
-    return $self->{ua};
-}
-
-# URL encode a string
-sub _encode_url {
-    my $str = shift;
-    $str =~ s/([^A-Za-z0-9_])/$1 eq ' ' ? '+' : sprintf("%%%02x", ord($1))/eg;
-    return $str;
-}
-
-# Turn a hash reference into a query string.
-sub _encode_query {
-    my $hash = shift || {};
-    return join '&',
-      map { _encode_url( $_ ) . '=' . _encode_url( $hash->{$_} ) }
-      sort keys %$hash;
-}
-
-# (X)HTML entity encode a string
-sub _encode_entity {
-    my $str = shift;
-    $str =~ s/$ENT_RE/$ENT_MAP{$1}/eg;
-    return $str;
-}
-
-# Generate an opening (X)HTML tag
-sub _open_tag {
-    my $name   = shift;
-    my $attr   = shift || {};
-    my $closed = shift;
-
-    return "<$name"
-      . join( '',
-        map { ' ' . $_ . '="' . _encode_entity( $attr->{$_} ) . '"' }
-          sort keys %$attr )
-      . ( $closed ? ' />' : '>' );
-}
-
-# Generate a closing (X)HTML tag
-sub _close_tag {
-    my $name = shift;
-    return "</$name>";
-}
-
-# Minimal JSON encoder
-sub _json_lite {
-    my $obj = shift;
-    if ( my $type = ref $obj ) {
-        if ( 'HASH' eq $type ) {
-            return '{'
-              . join( ',',
-                map { _json_lite( $_ ) . ':' . _json_lite( $obj->{$_} ) }
-                  sort keys %$obj )
-              . '}';
-        }
-        elsif ( 'ARRAY' eq $type ) {
-            return '[' . join( ',', map { _json_lite( $_ ) } @$obj ) . ']';
-        }
-        else {
-            croak "Can't convert a $type to JSON";
-        }
-    }
-    else {
-        if ( $obj =~ /^-?\d+(?:[.]\d+)?$/ ) {
-            return $obj;
-        }
-        else {
-            $obj =~ s/$JSON_RE/$JSON_ESCAPE{$1}/eg;
-            return '"' . $obj . '"';
-        }
-    }
-}
-
 sub get_options_setter {
-    my $self = shift;
+    my $self    = shift;
     my $options = shift || return '';
-    return join( '',
-        _open_tag( 'script', { type => 'text/javascript' } ),
-        "\n//<![CDATA[\n",
-        "var RecaptchaOptions = ",
-        _json_lite( $options ),
-        ";\n",
-        "//]]>\n",
-        _close_tag( 'script' ),
-        "\n" );
+    my $h       = HTML::Tiny->new();
+
+    return $h->script(
+        { type => 'text/javascript' },
+        "\n//<![CDATA[\n"
+          . "var RecaptchaOptions = "
+          . $h->json_encode( $options )
+          . ";\n//]]>\n"
+    ) . "\n";
 }
 
 sub get_html {
@@ -161,52 +52,57 @@ sub get_html {
       "To use reCAPTCHA you must get an API key from http://recaptcha.net/api/getkey"
       unless $pubkey;
 
-    _check_key( 'MAIN_KEY', $pubkey );
-
+    my $h = HTML::Tiny->new();
     my $server = $use_ssl ? API_SECURE_SERVER : API_SERVER;
 
     my $query = { k => $pubkey };
-    $query->{error} = $error if $error;
-    my $qs = _encode_query( $query );
+    if ( $error ) {
+        # Handle the case where the result hash from check_answer
+        # is passed.
+        if ( 'HASH' eq ref $error ) {
+            return '' if $error->{is_valid};
+            $error = $error->{error};
+        }
+        $query->{error} = $error;
+    }
+    my $qs = $h->query_encode( $query );
 
     return join(
         '',
         $self->get_options_setter( $options ),
-        _open_tag(
-            'script',
+        $h->script(
             {
                 type => 'text/javascript',
                 src  => "$server/challenge?$qs",
             }
         ),
-        _close_tag( 'script' ),
         "\n",
-        _open_tag( 'noscript' ),
-        _open_tag(
-            'iframe',
-            {
-                src         => "$server/noscript?$qs",
-                height      => 300,
-                width       => 500,
-                frameborder => 0
-            }
+        $h->noscript(
+            [
+                $h->iframe(
+                    {
+                        src         => "$server/noscript?$qs",
+                        height      => 300,
+                        width       => 500,
+                        frameborder => 0
+                    }
+                ),
+                $h->textarea(
+                    {
+                        name => 'recaptcha_challenge_field',
+                        rows => 3,
+                        cols => 40
+                    }
+                ),
+                $h->input(
+                    {
+                        type  => 'hidden',
+                        name  => 'recaptcha_response_field',
+                        value => 'manual_challenge'
+                    }
+                )
+            ]
         ),
-        _close_tag( 'iframe' ),
-        _open_tag(
-            'textarea',
-            { name => 'recaptcha_challenge_field', rows => 3, cols => 40 }
-        ),
-        _close_tag( 'textarea' ),
-        _open_tag(
-            'input',
-            {
-                type  => 'hidden',
-                name  => 'recaptcha_response_field',
-                value => 'manual_challenge'
-            },
-            1
-        ),
-        _close_tag( 'noscript' ),
         "\n"
     );
 }
@@ -215,7 +111,8 @@ sub _post_request {
     my $self = shift;
     my ( $url, $args ) = @_;
 
-    return $self->_get_ua->post( $url, $args );
+    my $ua = LWP::UserAgent->new();
+    return $ua->post( $url, $args );
 }
 
 sub check_answer {
@@ -226,8 +123,6 @@ sub check_answer {
       "To use reCAPTCHA you must get an API key from http://recaptcha.net/api/getkey"
       unless $privkey;
 
-    _check_key( 'MAIN_KEY', $privkey );
-
     croak "For security reasons, you must pass the remote ip to reCAPTCHA"
       unless $remoteip;
 
@@ -259,98 +154,6 @@ sub check_answer {
     }
 }
 
-sub _aes_encrypt {
-    my ( $val, $ky ) = @_;
-
-    my $val_len = length( $val );
-    my $pad_len = int( ( $val_len + 15 ) / 16 ) * 16;
-
-    # Pad value
-    $val .= chr( 16 - $val_len % 16 ) x ( $pad_len - $val_len )
-      if $val_len < $pad_len;
-
-    my $cipher = Crypt::Rijndael->new( $ky, Crypt::Rijndael::MODE_CBC );
-    $cipher->set_iv( "\0" x 16 );
-
-    return $cipher->encrypt( $val );
-}
-
-sub _urlbase64 {
-    my $str = shift;
-    chomp( my $enc = encode_base64( $str ) );
-    $enc =~ tr{+/}{-_};
-    return $enc;
-}
-
-sub mailhide_url {
-    my $self = shift;
-    my ( $pubkey, $privkey, $email ) = @_;
-
-    croak "To use reCAPTCHA Mailhide, you have to sign up for a public and "
-      . "private key. You can do so at http://mailhide.recaptcha.net/apikey."
-      unless $pubkey && $privkey;
-
-    _check_key( 'MAILHIDE_PUBLIC',  $pubkey );
-    _check_key( 'MAILHIDE_PRIVATE', $privkey );
-
-    croak "You must supply an email address"
-      unless $email;
-
-    return API_MAILHIDE_SERVER . '/d?'
-      . _encode_query(
-        {
-            k => $pubkey,
-            c => _urlbase64( _aes_encrypt( $email, pack( 'H*', $privkey ) ) )
-        }
-      );
-}
-
-sub _email_parts {
-    my ( $user, $dom ) = split( /\@/, shift, 2 );
-    my $ul = length( $user );
-    return ( substr( $user, 0, $ul <= 4 ? 1 : $ul <= 6 ? 3 : 4 ), '...', '@',
-        $dom );
-}
-
-sub mailhide_html {
-    my $self = shift;
-    my ( $pubkey, $privkey, $email ) = @_;
-
-    my $url = $self->mailhide_url( $pubkey, $privkey, $email );
-    my ( $user, $dots, $at, $dom ) = _email_parts( $email );
-
-    my %window_options = (
-        toolbar    => 0,
-        scrollbars => 0,
-        location   => 0,
-        statusbar  => 0,
-        menubar    => 0,
-        resizable  => 0,
-        width      => 500,
-        height     => 300
-    );
-
-    my $options = join ',',
-      map { "$_=$window_options{$_}" } sort keys %window_options;
-
-    return join(
-        '',
-        _encode_entity( $user ),
-        _open_tag(
-            'a',
-            {
-                href    => $url,
-                onclick => "window.open('$url', '', '$options'); return false;",
-                title   => 'Reveal this e-mail address'
-            }
-        ),
-        $dots,
-        _close_tag( 'a' ),
-        $at,
-        _encode_entity( $dom )
-    );
-}
-
 1;
 __END__
 
@@ -360,7 +163,7 @@ Captcha::reCAPTCHA - A Perl implementation of the reCAPTCHA API
 
 =head1 VERSION
 
-This document describes Captcha::reCAPTCHA version 0.6
+This document describes Captcha::reCAPTCHA version 0.7
 
 =head1 SYNOPSIS
 
@@ -406,6 +209,10 @@ found here:
 
 L<http://recaptcha.net/plugins/php/>
 
+To use reCAPTCHA you need to register your site here:
+
+L<https://admin.recaptcha.net/recaptcha/createsite/>
+
 =head1 INTERFACE
 
 =over
@@ -414,16 +221,6 @@ L<http://recaptcha.net/plugins/php/>
 
 Create a new C<< Captcha::reCAPTCHA >>.
 
-=back
-
-=head2 reCAPTCHA
-
-To use reCAPTCHA you need to register your site here:
-
-L<https://admin.recaptcha.net/recaptcha/createsite/>
-
-=over
-
 =item C<< get_html( $pubkey, $error, $use_ssl, $options ) >>
 
 Generates HTML to display the captcha.
@@ -438,8 +235,8 @@ Your reCAPTCHA public key, from the API Signup Page
 
 =item C<< $error >>
 
-Optional. If this string is set, the reCAPTCHA area will display the error code
-given. This error code comes from $response->{error}.
+Optional. If set this should be either a string containing a reCAPTCHA
+status code or a result hash as returned by C<< check_answer >>.
 
 =item C<< $use_ssl >>
 
@@ -530,59 +327,6 @@ See the /examples subdirectory for examples of how to call C<check_answer>.
 
 =back
 
-=head2 reCAPTCHA Mailhide
-
-To use reCAPTCHA Mailhide you need to get a public, private key pair
-from this page:
-
-L<http://mailhide.recaptcha.net/apikey>
-
-The Mailhide API consists of two methods C<< mailhide_html >>
-and C<< mailhide_url >>. The methods have the same parameters.
-
-The _html version returns HTML that can be directly put on your web
-page. The username portion of the email that is passed in is
-truncated and replaced with a link that calls Mailhide. The _url
-version gives you the url to decode the email and leaves it up to you
-to place the email in HTML.
-
-=over
-
-=item C<< mailhide_url( $pubkey, $privkey, $email ) >>
-
-Generate a link that will decode the specified email address.
-
-=over
-
-=item C<< $pubkey >>
-
-The Mailhide public key from the signup page
-
-=item C<< $privkey >>
-
-The Mailhide private key from the signup page
-
-=item C<< $email >>
-
-The email address you want to hide.
-
-=back
-
-Returns a URL that when clicked will allow the user to decode the hidden
-email address.
-
-=item C<< mailhide_html( $pubkey, $privkey, $email ) >>
-
-Generates HTML markup to embed a Mailhide protected email address
-on a page.
-
-The arguments are the same as for C<mailhide_url>.
-
-Returns a string containing HTML that may be embedded directly in
-a web page.
-
-=back
-
 =head1 CONFIGURATION AND ENVIRONMENT
 
 Captcha::reCAPTCHA requires no configuration files or environment
@@ -592,15 +336,10 @@ To use reCAPTCHA sign up for a key pair here:
 
 L<https://admin.recaptcha.net/recaptcha/createsite/>
 
-To use Mailhide get a public/private key pair here:
-
-L<http://mailhide.recaptcha.net/apikey>
-
 =head1 DEPENDENCIES
 
 LWP::UserAgent,
-Crypt::Rijndael,
-MIME::Base64,
+HTML::Tiny
 
 =head1 INCOMPATIBILITIES
 
@@ -608,8 +347,6 @@ None reported .
 
 =head1 BUGS AND LIMITATIONS
 
-Doesn't currently implement Mailhide support.
-
 No bugs have been reported.
 
 Please report any bugs or feature requests to
